@@ -17,6 +17,7 @@ gi.require_version('Notify', '0.7')
 gi.require_version('TotemPlParser', '1.0')
 from gi.repository import Gtk, Gio, GLib, Gdk, Gst, Notify, TotemPlParser
 
+from pickle import dump
 from locale import getlocale
 from gettext import gettext as _
 from gettext import ngettext as ngettext
@@ -35,14 +36,14 @@ except Exception as e:
     LastFM = None
 
 from lollypop.utils import is_gnome, is_unity
-from lollypop.define import ArtSize, Type
+from lollypop.define import ArtSize, Type, DataPath
 from lollypop.window import Window
 from lollypop.database import Database
 from lollypop.player import Player
 from lollypop.art import Art
+
 from lollypop.sqlcursor import SqlCursor
 from lollypop.settings import Settings, SettingsDialog
-from lollypop.mpris import MPRIS
 from lollypop.notification import NotificationManager
 from lollypop.database_albums import AlbumsDatabase
 from lollypop.database_artists import ArtistsDatabase
@@ -54,6 +55,16 @@ from lollypop.collectionscanner import CollectionScanner
 from lollypop.fullscreen import FullScreen
 
 
+# Ubuntu > 16.04
+if Gtk.get_minor_version() > 18:
+    from lollypop.mpris import MPRIS
+    from lollypop.inhibitor import Inhibitor
+# Ubuntu <= 16.04, Debian Jessie, ElementaryOS
+else:
+    from lollypop.mpris_legacy import MPRIS
+    from lollypop.inhibitor_legacy import Inhibitor
+
+
 class Application(Gtk.Application):
     """
         Lollypop application:
@@ -110,8 +121,12 @@ class Application(Gtk.Application):
         """
             Init main application
         """
-        cssProviderFile = Gio.File.new_for_uri(
-            'resource:///org/gnome/Lollypop/application.css')
+        if Gtk.get_minor_version() > 18:
+            cssProviderFile = Gio.File.new_for_uri(
+                'resource:///org/gnome/Lollypop/application.css')
+        else:
+            cssProviderFile = Gio.File.new_for_uri(
+                'resource:///org/gnome/Lollypop/application-legacy.css')
         cssProvider = Gtk.CssProvider()
         cssProvider.load_from_file(cssProviderFile)
         screen = Gdk.Screen.get_default()
@@ -120,8 +135,6 @@ class Application(Gtk.Application):
                                              Gtk.STYLE_PROVIDER_PRIORITY_USER)
         self.settings = Settings.new()
         ArtSize.BIG = self.settings.get_value('cover-size').get_int32()
-        if LastFM is not None:
-            self.lastfm = LastFM()
         self.db = Database()
         self.playlists = Playlists()
         # We store cursors for main thread
@@ -134,6 +147,8 @@ class Application(Gtk.Application):
         self.player = Player()
         self.scanner = CollectionScanner()
         self.art = Art()
+        if LastFM is not None:
+            self.lastfm = LastFM()
         if not self.settings.get_value('disable-mpris'):
             MPRIS(self)
         if not self.settings.get_value('disable-notifications'):
@@ -180,6 +195,11 @@ class Application(Gtk.Application):
             self.window.init_list_one()
             self.window.show()
             self.player.restore_state()
+            # We add to mainloop as we want to run
+            # after player::restore_state() signals
+            GLib.idle_add(self.window.toolbar.set_mark)
+            # Will not start sooner
+            self.inhibitor = Inhibitor()
 
     def prepare_to_exit(self, action=None, param=None):
         """
@@ -195,11 +215,21 @@ class Application(Gtk.Application):
             elif self.player.current_track.id == Type.RADIOS:
                 radios = Radios()
                 track_id = radios.get_id(
-                                        self.player.current_track.album_artist)
+                                    self.player.current_track.album_artists[0])
             else:
                 track_id = self.player.current_track.id
-            self.settings.set_value('track-id', GLib.Variant('i',
-                                                             track_id))
+                # Save albums context
+                try:
+                    dump(self.player.context.genre_ids,
+                         open(DataPath + "/genre_ids.bin", "wb"))
+                    dump(self.player.context.artist_ids,
+                         open(DataPath + "/artist_ids.bin", "wb"))
+                    self.player.shuffle_albums(False)
+                    dump(self.player.get_albums(),
+                         open(DataPath + "/albums.bin", "wb"))
+                except Exception as e:
+                    print("Application::prepare_to_exit()", e)
+            dump(track_id, open(DataPath + "/track_id.bin", "wb"))
             # Save current playlist
             if self.player.current_track.id == Type.RADIOS:
                 playlist_ids = [Type.RADIOS]
@@ -207,8 +237,13 @@ class Application(Gtk.Application):
                 playlist_ids = []
             else:
                 playlist_ids = self.player.get_user_playlist_ids()
-            self.settings.set_value('playlist-ids', GLib.Variant('ai',
-                                                                 playlist_ids))
+            dump(playlist_ids,
+                 open(DataPath + "/playlist_ids.bin", "wb"))
+        if self.player.is_playing():
+            position = self.player.position
+        else:
+            position = 0
+        dump(position, open(DataPath + "/position.bin", "wb"))
         self.player.stop_all()
         if self.window:
             self.window.stop_all()
@@ -422,12 +457,10 @@ class Application(Gtk.Application):
 
         menu = builder.get_object('app-menu')
 
-        # TODO: Remove this test later
-        if Gtk.get_minor_version() > 12:
-            settingsAction = Gio.SimpleAction.new('settings', None)
-            settingsAction.connect('activate', self._settings_dialog)
-            self.set_accels_for_action('app.settings', ["<Control>s"])
-            self.add_action(settingsAction)
+        settingsAction = Gio.SimpleAction.new('settings', None)
+        settingsAction.connect('activate', self._settings_dialog)
+        self.set_accels_for_action('app.settings', ["<Control>s"])
+        self.add_action(settingsAction)
 
         updateAction = Gio.SimpleAction.new('update_db', None)
         updateAction.connect('activate', self._update_db)
