@@ -16,6 +16,7 @@ import itertools
 
 from lollypop.sqlcursor import SqlCursor
 from lollypop.define import Lp, Type
+from lollypop.utils import remove_static_genres
 
 
 class AlbumsDatabase:
@@ -29,13 +30,11 @@ class AlbumsDatabase:
         """
         self._cached_randoms = []
 
-    def add(self, name, artist_id, no_album_artist, year,
-            path, popularity, mtime):
+    def add(self, name, artist_ids, year, path, popularity, mtime):
         """
             Add a new album to database
             @param Album name as string
-            @param artist id as int,
-            @param no_album_artist as bool,
+            @param artist ids as int
             @param year as int
             @param path as string
             @param mtime as int
@@ -44,13 +43,31 @@ class AlbumsDatabase:
         """
         with SqlCursor(Lp().db) as sql:
             result = sql.execute("INSERT INTO albums\
-                                  (name, artist_id, no_album_artist, year,\
+                                  (name, no_album_artist, year,\
                                   path, popularity, mtime)\
-                                  VALUES (?, ?, ?, ?, ?, ?, ?)",
-                                 (name, artist_id, no_album_artist, year,
+                                  VALUES (?, ?, ?, ?, ?, ?)",
+                                 (name, artist_ids == [], year,
                                   path, popularity, mtime))
+            for artist_id in artist_ids:
+                sql.execute("INSERT INTO album_artists\
+                             (album_id, artist_id)\
+                             VALUES (?, ?)", (result.lastrowid, artist_id))
             return result.lastrowid
 
+    def add_artist(self, album_id, artist_id):
+        """
+            Add artist to track
+            @param album id as int
+            @param artist id as int
+            @warning: commit needed
+        """
+        with SqlCursor(Lp().db) as sql:
+            artist_ids = self.get_artist_ids(album_id)
+            if artist_id not in artist_ids:
+                sql.execute("INSERT INTO "
+                            "album_artists (album_id, artist_id)"
+                            "VALUES (?, ?)", (album_id, artist_id))
+
     def add_genre(self, album_id, genre_id):
         """
             Add genre to album
@@ -65,15 +82,22 @@ class AlbumsDatabase:
                             "album_genres (album_id, genre_id)"
                             "VALUES (?, ?)", (album_id, genre_id))
 
-    def set_artist_id(self, album_id, artist_id):
+    def set_artist_ids(self, album_id, artist_ids):
         """
             Set artist id
-            @param album id as int, artist_id as int
+            @param album id as int
+            @param artist_ids as [int]
             @warning: commit needed
         """
         with SqlCursor(Lp().db) as sql:
-            sql.execute("UPDATE albums SET artist_id=? WHERE rowid=?",
-                        (artist_id, album_id))
+            currents = self.get_artist_ids(album_id)
+            if not currents or set(currents) - set(artist_ids):
+                sql.execute("DELETE FROM album_artists\
+                            WHERE album_id=?", (album_id,))
+                for artist_id in artist_ids:
+                    sql.execute("INSERT INTO album_artists\
+                                (album_id, artist_id)\
+                                VALUES (?, ?)", (album_id, artist_id))
 
     def set_year(self, album_id, year):
         """
@@ -185,42 +209,43 @@ class AlbumsDatabase:
             @return Album id as int
         """
         with SqlCursor(Lp().db) as sql:
+            filters = (album_name, artist_id)
+            request = "SELECT rowid FROM albums where name=?\
+                       AND artist_id=? "
             if year is None:
-                result = sql.execute("SELECT rowid FROM albums where name=?\
-                                      AND artist_id=?\
-                                      AND year is null",
-                                     (album_name, artist_id))
+                request += "AND year is null"
             else:
-                result = sql.execute("SELECT rowid FROM albums where name=?\
-                                      AND artist_id=?\
-                                      AND year =?",
-                                     (album_name, artist_id, year))
+                request += "AND year =?"
+                filters += (year,)
+            result = sql.execute(request, filters)
             v = result.fetchone()
             if v is not None:
                 return v[0]
             return None
 
-    def get_non_compilation_id(self, album_name, artist_id, year):
+    def get_non_compilation_id(self, album_name, artist_ids, year):
         """
             Get non compilation album id
             @param Album name as string,
-            @param artist id as int
+            @param artist ids as [int]
             @param year as int
             @return Album id as int
         """
         with SqlCursor(Lp().db) as sql:
+            filters = (album_name,)
+            filters += tuple(artist_ids)
+            request = "SELECT albums.rowid FROM albums, album_artists\
+                       WHERE name=? AND\
+                       no_album_artist=0 AND\
+                       album_artists.album_id=albums.rowid AND (1=0 "
+            for artist_id in artist_ids:
+                request += "OR artist_id=? "
             if year is None:
-                result = sql.execute("SELECT rowid FROM albums where name=?\
-                                      AND artist_id=?\
-                                      AND year is null\
-                                      AND no_album_artist=0",
-                                     (album_name, artist_id))
+                request += ") AND year is null"
             else:
-                result = sql.execute("SELECT rowid FROM albums where name=?\
-                                      AND artist_id=?\
-                                      AND year =?\
-                                      AND no_album_artist=0",
-                                     (album_name, artist_id, year))
+                filters += (year,)
+                request += ") AND year=?"
+            result = sql.execute(request, filters)
             v = result.fetchone()
             if v is not None:
                 return v[0]
@@ -273,35 +298,32 @@ class AlbumsDatabase:
 
             return _("Unknown")
 
-    def get_artist_name(self, album_id):
+    def get_artists(self, album_id):
         """
-            Get artist name
+            Get artist names
             @param Album id as int
-            @return Artist name as string
+            @return artists as [str]
         """
         with SqlCursor(Lp().db) as sql:
-            result = sql.execute("SELECT artists.name from artists, albums\
-                                  WHERE albums.rowid=? AND albums.artist_id ==\
-                                  artists.rowid", (album_id,))
-            v = result.fetchone()
-            if v is not None:
-                return v[0]
+            result = sql.execute("SELECT artists.name\
+                                 FROM artists, album_artists\
+                                 WHERE album_artists.album_id=?\
+                                 AND album_artists.artist_id=artists.rowid",
+                                 (album_id,))
+            return list(itertools.chain(*result))
 
-        return _("Compilation")
-
-    def get_artist_id(self, album_id):
+    def get_artist_ids(self, album_id):
         """
             Get album artist id
             @param album_id
-            @return artist id
+            @return artist ids as [int]
         """
         with SqlCursor(Lp().db) as sql:
-            result = sql.execute("SELECT artist_id FROM albums where rowid=?",
+            result = sql.execute("SELECT artist_id\
+                                  FROM album_artists\
+                                  WHERE album_id=?",
                                  (album_id,))
-            v = result.fetchone()
-            if v is not None:
-                return v[0]
-            return None
+            return list(itertools.chain(*result))
 
     def get_year(self, album_id):
         """
@@ -331,7 +353,7 @@ class AlbumsDatabase:
             if v is not None:
                 path = v[0]
             if path != "" and not os.path.exists(path):
-                tracks = self.get_tracks(album_id, None)
+                tracks = self.get_tracks(album_id, [], [])
                 if tracks:
                     filepath = Lp().tracks.get_path(tracks[0])
                     path = os.path.dirname(filepath)
@@ -422,22 +444,20 @@ class AlbumsDatabase:
             @param genre ids as [int]
             @return count as int
         """
+        genre_ids = remove_static_genres(genre_ids)
         with SqlCursor(Lp().db) as sql:
-            if genre_ids and genre_ids[0] > 0:
-                filters = (album_id,)
-                filters += tuple(genre_ids)
-                request = "SELECT COUNT(1)\
-                           FROM tracks, track_genres\
-                           WHERE tracks.album_id=?\
-                           AND track_genres.track_id = tracks.rowid AND ("
+            filters = (album_id,)
+            filters += tuple(genre_ids)
+            request = "SELECT COUNT(1)\
+                       FROM tracks, track_genres\
+                       WHERE tracks.album_id=?\
+                       AND track_genres.track_id = tracks.rowid"
+            if genre_ids:
+                request += " AND ("
                 for genre_id in genre_ids:
                     request += "track_genres.genre_id=? OR "
                 request += "1=0)"
-                result = sql.execute(request, filters)
-            else:
-                result = sql.execute("SELECT COUNT(1)\
-                                      FROM tracks\
-                                      WHERE tracks.album_id=?", (album_id,))
+            result = sql.execute(request, filters)
             v = result.fetchone()
             if v is not None:
                 return v[0]
@@ -450,110 +470,143 @@ class AlbumsDatabase:
             @param genre id as int
             @return [disc as int]
         """
+        genre_ids = remove_static_genres(genre_ids)
         with SqlCursor(Lp().db) as sql:
-            if genre_ids and genre_ids[0] > 0:
-                filters = (album_id,)
-                filters += tuple(genre_ids)
-                request = "SELECT DISTINCT discnumber\
-                           FROM tracks, track_genres\
-                           WHERE tracks.album_id=?\
-                           AND track_genres.track_id = tracks.rowid AND ("
+            filters = (album_id,)
+            filters += tuple(genre_ids)
+            request = "SELECT DISTINCT discnumber\
+                       FROM tracks, track_genres\
+                       WHERE tracks.album_id=?\
+                       AND track_genres.track_id = tracks.rowid"
+            if genre_ids:
+                request += " AND ("
                 for genre_id in genre_ids:
                     request += "track_genres.genre_id=? OR "
-                request += "1=0) ORDER BY discnumber"
-                result = sql.execute(request, filters)
-            else:
-                result = sql.execute("SELECT DISTINCT discnumber\
-                                      FROM tracks\
-                                      WHERE tracks.album_id=?\
-                                      ORDER BY discnumber", (album_id,))
+                request += "1=0)"
+            request += " ORDER BY discnumber"
+            result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
-    def get_tracks(self, album_id, genre_ids):
+    def get_tracks(self, album_id, genre_ids, artist_ids):
         """
             Get tracks for album id
             @param album id as int
             @param genre ids as [int]
-            @return Arrays of tracks id as int
+            @param artist_ids as [int]
+            @return track ids as [int]
         """
+        genre_ids = remove_static_genres(genre_ids)
+        # Reset filters if not needed
+        if not self._has_genres(album_id):
+            genre_ids = []
+        if not self._has_artists(album_id):
+            artist_ids = []
         with SqlCursor(Lp().db) as sql:
-            if genre_ids and genre_ids[0] > 0:
-                filters = (album_id,)
+            filters = (album_id,)
+            request = "SELECT DISTINCT tracks.rowid\
+                       FROM tracks"
+            if genre_ids:
+                request += ", track_genres"
                 filters += tuple(genre_ids)
-                request = "SELECT tracks.rowid\
-                           FROM tracks, track_genres\
-                           WHERE album_id=?\
-                           AND track_genres.track_id = tracks.rowid AND ("
+            if artist_ids:
+                request += ", track_artists"
+                filters += tuple(artist_ids)
+            request += " WHERE album_id=? "
+            if genre_ids:
+                request += "AND track_genres.track_id=tracks.rowid AND ("
                 for genre_id in genre_ids:
                     request += "track_genres.genre_id=? OR "
-                request += "1=0) ORDER BY discnumber, tracknumber"
-                result = sql.execute(request, filters)
-            else:
-                result = sql.execute("SELECT rowid FROM tracks\
-                                      WHERE album_id=?\
-                                      ORDER BY discnumber, tracknumber",
-                                     (album_id,))
+                request += "1=0)"
+            if artist_ids:
+                request += "AND track_artists.track_id=tracks.rowid AND ("
+                for artist_id in artist_ids:
+                    request += "track_artists.artist_id=? OR "
+                request += "1=0)"
+            request += " ORDER BY discnumber, tracknumber"
+            result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
-    def get_tracks_path(self, album_id, genre_ids):
+    def get_tracks_path(self, album_id, genre_ids, artist_ids):
         """
             Get tracks path for album id/disc
             Will search track from albums from same artist
             with same name and different genre
             @param album id as int
             @param genre ids as [int]
+            @param artist_ids as [int]
             @return Arrays of tracks id as int
         """
+        genre_ids = remove_static_genres(genre_ids)
+        # Reset filters if not needed
+        if not self._has_genres(album_id):
+            genre_ids = []
+        if not self._has_artists(album_id):
+            artist_ids = []
         with SqlCursor(Lp().db) as sql:
-            if genre_ids and genre_ids[0] > 0:
-                filters = (album_id,)
+            filters = (album_id,)
+            request = "SELECT DISTINCT tracks.filepath\
+                       FROM tracks"
+            if genre_ids:
+                request += ", track_genres"
                 filters += tuple(genre_ids)
-                request = "SELECT tracks.filepath\
-                           FROM tracks, track_genres\
-                           WHERE album_id=?\
-                           AND track_genres.track_id = tracks.rowid AND ("
+            if artist_ids:
+                request += ", track_artists"
+                filters += tuple(artist_ids)
+            request += " WHERE album_id=? "
+            if genre_ids:
+                request += "AND track_genres.track_id = tracks.rowid AND ("
                 for genre_id in genre_ids:
                     request += "track_genres.genre_id=? OR "
-                request += "1=0) ORDER BY discnumber, tracknumber"
-                result = sql.execute(request, filters)
-            else:
-                result = sql.execute("SELECT tracks.filepath\
-                                      FROM tracks\
-                                      WHERE album_id=?\
-                                      ORDER BY discnumber, tracknumber",
-                                     (album_id,))
+                request += "1=0)"
+            if artist_ids:
+                request += "AND track_artists.track_id=tracks.rowid AND ("
+                for artist_id in artist_ids:
+                    request += "track_artists.artist_id=? OR "
+                request += "1=0)"
+            request += " ORDER BY discnumber, tracknumber"
+            result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
-    def get_disc_tracks(self, album_id, genre_ids, disc):
+    def get_disc_tracks(self, album_id, genre_ids, artist_ids, disc):
         """
             Get tracks ids for album id disc
 
-            @param album_id as int
-            @param genre_ids as [int]
+            @param album id as int
+            @param genre ids as [int]
+            @param artist ids as [int]
             @param disc as int
             @return [int]
         """
+        genre_ids = remove_static_genres(genre_ids)
+        # Reset filters if not needed
+        if not self._has_genres(album_id):
+            genre_ids = []
+        if not self._has_artists(album_id):
+            artist_ids = []
         with SqlCursor(Lp().db) as sql:
-            if genre_ids and genre_ids[0] > 0:
-                filters = (album_id, disc)
+            filters = (album_id, disc)
+            request = "SELECT DISTINCT tracks.rowid\
+                       FROM tracks"
+            if genre_ids:
+                request += ", track_genres"
                 filters += tuple(genre_ids)
-                request = "SELECT tracks.rowid\
-                           FROM tracks, track_genres\
-                           WHERE album_id=?\
-                           AND discnumber=?\
-                           AND track_genres.track_id = tracks.rowid AND ("
+            if artist_ids:
+                request += ", track_artists"
+                filters += tuple(artist_ids)
+            request += " WHERE album_id=?\
+                       AND discnumber=?"
+            if genre_ids:
+                request += " AND track_genres.track_id = tracks.rowid AND ("
                 for genre_id in genre_ids:
                     request += "track_genres.genre_id=? OR "
-                request += "1=0) ORDER BY discnumber, tracknumber"
-                result = sql.execute(request, filters)
-            else:
-                result = sql.execute("SELECT tracks.rowid\
-                                      FROM tracks\
-                                      WHERE tracks.album_id = ?\
-                                      AND discnumber=?\
-                                      ORDER BY discnumber, tracknumber",
-                                     (album_id, disc))
-
+                request += "1=0)"
+            if artist_ids:
+                request += " AND track_artists.track_id=tracks.rowid AND ("
+                for artist_id in artist_ids:
+                    request += "track_artists.artist_id=? OR "
+                request += "1=0)"
+            request += " ORDER BY discnumber, tracknumber"
+            result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
     def get_ids(self, artist_ids=[], genre_ids=[]):
@@ -563,62 +616,65 @@ class AlbumsDatabase:
             @param genre ids as [int]
             @return Array of album ids as int
         """
+        genre_ids = remove_static_genres(genre_ids)
         with SqlCursor(Lp().db) as sql:
             result = []
             # Get albums for all artists
             if not artist_ids and not genre_ids:
                 result = sql.execute(
                                  "SELECT DISTINCT albums.rowid\
-                                  FROM albums, artists\
-                                  WHERE artists.rowid=albums.artist_id\
-                                  ORDER BY artists.sortname COLLATE NOCASE,\
+                                  FROM albums, artists, album_artists\
+                                  WHERE artists.rowid=album_artists.artist_id\
+                                  AND albums.rowid=album_artists.album_id\
+                                  ORDER BY artists.sortname\
+                                  COLLATE NOCASE COLLATE LOCALIZED,\
                                   albums.year,\
-                                  albums.name COLLATE NOCASE")
+                                  albums.name\
+                                  COLLATE NOCASE COLLATE LOCALIZED")
             # Get albums for genre
             elif not artist_ids:
                 genres = tuple(genre_ids)
                 request = "SELECT DISTINCT albums.rowid FROM albums,\
-                           album_genres, artists\
-                           WHERE artists.rowid=artist_id\
+                           album_genres, artists, album_artists\
+                           WHERE artists.rowid=album_artists.artist_id\
+                           AND albums.rowid=album_artists.album_id\
                            AND album_genres.album_id=albums.rowid AND ("
                 for genre_id in genre_ids:
                     request += "album_genres.genre_id=? OR "
-                request += "1=0) ORDER BY artists.sortname COLLATE NOCASE,\
+                request += "1=0) ORDER BY artists.sortname\
+                            COLLATE NOCASE COLLATE LOCALIZED,\
                             albums.year,\
-                            albums.name COLLATE NOCASE"
+                            albums.name COLLATE NOCASE COLLATE LOCALIZED"
                 result = sql.execute(request, genres)
             # Get albums for artist
             elif not genre_ids:
                 artists = tuple(artist_ids)
                 request = "SELECT DISTINCT albums.rowid\
-                           FROM albums, artists WHERE ("
+                           FROM albums, artists, album_artists WHERE\
+                           artists.rowid=album_artists.artist_id AND\
+                           album_artists.album_id=albums.rowid AND ("
                 for artist_id in artist_ids:
-                    request += "artist_id=? OR "
-                if len(artist_ids) > 1:
-                    request += "1=0) AND artists.rowid=albums.artist_id\
-                                ORDER BY artists.name COLLATE NOCASE,\
-                                year, albums.name COLLATE NOCASE"
-                else:
-                    request += "1=0) ORDER BY year, albums.name COLLATE NOCASE"
+                    request += "album_artists.artist_id=? OR "
+                request += "1=0) ORDER BY artists.name COLLATE NOCASE COLLATE LOCALIZED,\
+                            year, albums.name COLLATE NOCASE COLLATE LOCALIZED"
                 result = sql.execute(request, artists)
             # Get albums for artist id and genre id
             else:
                 filters = tuple(artist_ids)
                 filters += tuple(genre_ids)
                 request = "SELECT DISTINCT albums.rowid\
-                           FROM albums, album_genres, artists\
-                           WHERE album_genres.album_id=albums.rowid AND ("
+                           FROM albums, album_genres, artists, album_artists\
+                           WHERE album_genres.album_id=albums.rowid AND\
+                           artists.rowid=album_artists.artist_id AND\
+                           album_artists.album_id=albums.rowid AND ("
                 for artist_id in artist_ids:
-                    request += "albums.artist_id=? OR "
+                    request += "album_artists.artist_id=? OR "
                 request += "1=0) AND ("
                 for genre_id in genre_ids:
                     request += "album_genres.genre_id=? OR "
-                if len(artist_ids) > 1:
-                    request += "1=0) AND artists.rowid=albums.artist_id\
-                                ORDER BY artists.name COLLATE NOCASE,\
-                                year, albums.name COLLATE NOCASE"
-                else:
-                    request += "1=0) ORDER BY year, albums.name COLLATE NOCASE"
+                request += "1=0) ORDER BY artists.name\
+                            COLLATE NOCASE COLLATE LOCALIZED,\
+                            year, albums.name COLLATE NOCASE COLLATE LOCALIZED"
                 result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
@@ -632,8 +688,10 @@ class AlbumsDatabase:
             result = []
             # Get all compilations
             if not genre_ids or genre_ids[0] == Type.ALL:
-                result = sql.execute("SELECT albums.rowid FROM albums\
-                                      WHERE artist_id=?\
+                result = sql.execute("SELECT DISTINCT albums.rowid\
+                                      FROM albums, album_artists\
+                                      WHERE album_artists.artist_id=?\
+                                      AND album_artists.album_id=albums.rowid\
                                       ORDER BY albums.name, albums.year",
                                      (Type.COMPILATIONS,))
             # Get compilation for genre id
@@ -641,9 +699,10 @@ class AlbumsDatabase:
                 filters = (Type.COMPILATIONS,)
                 filters += tuple(genre_ids)
                 request = "SELECT DISTINCT albums.rowid\
-                           FROM albums, album_genres\
+                           FROM albums, album_genres, album_artists\
                            WHERE album_genres.album_id=albums.rowid\
-                           AND albums.artist_id=? AND ( "
+                           AND album_artists.album_id=albums.rowid\
+                           AND album_artists.artist_id=? AND ( "
                 for genre_id in genre_ids:
                     request += "album_genres.genre_id=? OR "
                 request += "1==0) ORDER BY albums.name,albums.year"
@@ -680,32 +739,36 @@ class AlbumsDatabase:
     def search(self, string):
         """
             Search for albums looking like string
-            @param string
-            return: Array of (id as int, artist_id as int)
+            @param search as str
+            @return album ids as [int]
         """
         with SqlCursor(Lp().db) as sql:
-            result = sql.execute("SELECT rowid, artist_id FROM albums\
+            result = sql.execute("SELECT albums.rowid\
+                                  FROM albums\
                                   WHERE name LIKE ?\
                                   LIMIT 25", ('%' + string + '%',))
-            return list(result)
+            return list(itertools.chain(*result))
 
-    def is_compilation(self, album_id):
+    def calculate_artist_ids(self, album_id):
         """
-            True if is a compilation
+            Calculate artist ids based on tracks
+            @WARNING Be sure album already have a track
             @param album id as int
-            @return is compilation as bool
+            @return artist_ids as [int]
         """
-        with SqlCursor(Lp().db) as sql:
-            result = sql.execute(
-                            "SELECT COUNT(DISTINCT track_artists.artist_id)\
-                             FROM tracks, track_artists\
-                             WHERE tracks.album_id=?\
-                             AND tracks.rowid = track_artists.track_id",
-                            (album_id,))
-            v = result.fetchone()
-            if v is not None:
-                return v[0] > 1
-            return False
+        try:
+            ret = []
+            for track_id in self.get_tracks(album_id, [], []):
+                artist_ids = Lp().tracks.get_artist_ids(track_id)
+                # Check if previous track and
+                # track do not have same artists
+                if ret:
+                    if not set(ret) & set(artist_ids):
+                        return [Type.COMPILATIONS]
+                ret = artist_ids
+        except Exception as e:
+            print("AlbumsDatabase::calculate_artist_ids()", e)
+        return ret
 
     def count(self):
         """
@@ -752,5 +815,43 @@ class AlbumsDatabase:
             # Album empty, remove it
             if not v:
                 ret = True
+                sql.execute("DELETE FROM album_artists\
+                            WHERE album_id=?",
+                            (album_id,))
                 sql.execute("DELETE FROM albums WHERE rowid=?", (album_id,))
             return ret
+
+#######################
+# PRIVATE             #
+#######################
+    def _has_genres(self, album_id):
+        """
+            Return True if album has more than one genre
+            @param album id as int
+            @return bool
+        """
+        with SqlCursor(Lp().db) as sql:
+            result = sql.execute("SELECT COUNT(*)\
+                                 FROM album_genres\
+                                 WHERE album_id=?\
+                                 LIMIT 2", (album_id,))
+            v = result.fetchone()
+            if v is not None:
+                return v[0] > 1
+        return False
+
+    def _has_artists(self, album_id):
+        """
+            Return True if album has more than one artist
+            @param album id as int
+            @return bool
+        """
+        with SqlCursor(Lp().db) as sql:
+            result = sql.execute("SELECT COUNT(*)\
+                                 FROM album_artists\
+                                 WHERE album_id=?\
+                                 LIMIT 2", (album_id,))
+            v = result.fetchone()
+            if v is not None:
+                return v[0] > 1
+        return False
