@@ -12,7 +12,8 @@
 
 import gi
 gi.require_version('Secret', '1')
-from gi.repository import GLib, Gio
+gi.require_version('Goa', '1.0')
+from gi.repository import GLib, Gio, Goa
 
 from gettext import gettext as _
 
@@ -45,9 +46,6 @@ class LastFM(LastFMNetwork):
        really have much option :).
     """
 
-    _API_KEY = '7a9619a850ccf7377c46cf233c51e3c6'
-    _API_SECRET = '9254319364d73bec6c59ace485a95c98'
-
     def __init__(self):
         """
             Init lastfm support
@@ -56,13 +54,20 @@ class LastFM(LastFMNetwork):
             self._settings = Gio.Settings.new('org.gnome.system.proxy.http')
         except:
             self._settings = None
-        LastFMNetwork.__init__(self,
-                               api_key=self._API_KEY,
-                               api_secret=self._API_SECRET)
         self._username = ''
         self._is_auth = False
         self._password = None
         self._check_for_proxy()
+        self._goa = self._get_goa_oauth()
+        if self._goa is not None:
+            self._API_KEY = self._goa.props.client_id
+            self._API_SECRET = self._goa.props.client_secret
+        else:
+            self._API_KEY = '7a9619a850ccf7377c46cf233c51e3c6'
+            self._API_SECRET = '9254319364d73bec6c59ace485a95c98'
+        LastFMNetwork.__init__(self,
+                               api_key=self._API_KEY,
+                               api_secret=self._API_SECRET)
         self.connect(None)
 
     def connect(self, password):
@@ -70,21 +75,26 @@ class LastFM(LastFMNetwork):
             Connect lastfm
             @param password as str/None
         """
-        if Secret is None or\
-                not Gio.NetworkMonitor.get_default().get_network_available():
-            return
-        self._username = Lp().settings.get_value('lastfm-login').get_string()
-        if password is None:
-            schema = Secret.Schema.new("org.gnome.Lollypop",
-                                       Secret.SchemaFlags.NONE,
-                                       SecretSchema)
-            Secret.password_lookup(schema, SecretAttributes, None,
-                                   self._on_password_lookup)
-        else:
-            t = Thread(target=self._connect, args=(self._username,
-                                                   password, True))
+        if self._goa:
+            t = Thread(target=self._connect, args=('', '', True))
             t.daemon = True
             t.start()
+        # Get username/password from GSettings/Secret
+        elif Secret is not None and\
+                Gio.NetworkMonitor.get_default().get_network_available():
+            self._username = Lp().settings.get_value(
+                                                   'lastfm-login').get_string()
+            if password is None:
+                schema = Secret.Schema.new("org.gnome.Lollypop",
+                                           Secret.SchemaFlags.NONE,
+                                           SecretSchema)
+                Secret.password_lookup(schema, SecretAttributes, None,
+                                       self._on_password_lookup)
+            else:
+                t = Thread(target=self._connect, args=(self._username,
+                                                       password, True))
+                t.daemon = True
+                t.start()
 
     def connect_sync(self, password):
         """
@@ -168,7 +178,7 @@ class LastFM(LastFMNetwork):
         """
         # Love the track on lastfm
         if Gio.NetworkMonitor.get_default().get_network_available() and\
-           self.is_auth():
+           self._is_auth:
             track = self.get_track(artist, title)
             try:
                 track.love()
@@ -184,23 +194,43 @@ class LastFM(LastFMNetwork):
         """
         # Love the track on lastfm
         if Gio.NetworkMonitor.get_default().get_network_available() and\
-           self.is_auth():
+           self._is_auth:
             track = self.get_track(artist, title)
             try:
                 track.unlove()
             except Exception as e:
                 print("Lastfm::unlove(): %s" % e)
 
+    @property
     def is_auth(self):
         """
-            Return True if valid authentication send
-            @return bool
+            True if valid authentication send
         """
         return self._is_auth
 
+    @property
+    def is_goa(self):
+        """
+            True if using Gnome Online Account
+        """
+        return self._goa is not None
+
 #######################
 # PRIVATE             #
 #######################
+    def _get_goa_oauth(self):
+        """
+            Init Gnome Online Account
+            @return get_oauth2_based()/None
+        """
+        try:
+            c = Goa.Client.new_sync()
+            for proxy in c.get_accounts():
+                if proxy.get_account().props.provider_name == "Last.fm":
+                    return proxy.get_oauth2_based()
+        except:
+            return None
+
     def _check_for_proxy(self):
         """
             Enable proxy if needed
@@ -221,24 +251,31 @@ class LastFM(LastFMNetwork):
             @thread safe
         """
         self._username = username
-        if password != '' and username != '':
+        if self._goa is not None or (password != '' and username != ''):
             self._is_auth = True
         else:
             self._is_auth = False
         try:
             self._check_for_proxy()
-            LastFMNetwork.__init__(
-                self,
-                api_key=self._API_KEY,
-                api_secret=self._API_SECRET,
-                username=Lp().settings.get_value('lastfm-login').get_string(),
-                password_hash=md5(password))
+            if self._goa is None:
+                LastFMNetwork.__init__(
+                    self,
+                    api_key=self._API_KEY,
+                    api_secret=self._API_SECRET,
+                    username=self._username,
+                    password_hash=md5(password))
+            else:
+                LastFMNetwork.__init__(
+                    self,
+                    api_key=self._API_KEY,
+                    api_secret=self._API_SECRET,
+                    session_key=self._goa.call_get_access_token_sync(None)[0])
             if populate_loved:
                 self._populate_loved_tracks()
         except Exception as e:
             print("Lastfm::_connect(): %s" % e)
 
-    def _scrobble(self, artist, album, title, timestamp, duration):
+    def _scrobble(self, artist, album, title, timestamp, duration, first=True):
         """
             Scrobble track
             @param artist as str
@@ -246,6 +283,7 @@ class LastFM(LastFMNetwork):
             @param album_title as str
             @param timestamp as int
             @param duration as int
+            @param first is internal
             @thread safe
         """
         debug("LastFM::_scrobble(): %s, %s, %s, %s, %s" % (artist,
@@ -259,10 +297,13 @@ class LastFM(LastFMNetwork):
                                    album=album,
                                    title=title,
                                    timestamp=timestamp)
-        except BadAuthenticationError:
+        except BadAuthenticationError as e:
             pass
-        except:
-            self._connect(self._username, self._password)
+        except Exception as e:
+            print("Lastfm::scrobble():", e)
+            # Scrobble sometimes fails
+            if first:
+                self._connect(self._username, self._password)
 
     def _now_playing(self, artist, album, title, duration, first=True):
         """
@@ -274,20 +315,21 @@ class LastFM(LastFMNetwork):
             @param first is internal
             @thread safe
         """
-        debug("LastFM::_now_playing(): %s, %s, %s, %s" % (artist,
-                                                          album,
-                                                          title,
-                                                          duration))
         try:
             LastFMNetwork.update_now_playing(self,
                                              artist=artist,
                                              album=album,
                                              title=title,
                                              duration=duration)
+            debug("LastFM::_now_playing(): %s, %s, %s, %s" % (artist,
+                                                              album,
+                                                              title,
+                                                              duration))
         except BadAuthenticationError:
             if Lp().notify is not None:
                 GLib.idle_add(Lp().notify.send, _("Wrong Last.fm credentials"))
-        except:
+        except Exception as e:
+            print("Lastfm::scrobble():", e)
             # now playing sometimes fails
             if first:
                 self._connect(self._username, self._password)
