@@ -67,7 +67,7 @@ class Base:
             radios = Radios()
             avg_popularity = radios.get_avg_popularity()
             if avg_popularity > 0:
-                popularity = radios.get_popularity(self._album_artist)
+                popularity = radios.get_popularity(self._album_artists[0])
         return popularity * 5 / avg_popularity + 0.5
 
     def set_popularity(self, popularity):
@@ -86,7 +86,7 @@ class Base:
                 radios = Radios()
                 avg_popularity = radios.get_avg_popularity()
                 popularity = int((popularity * avg_popularity / 5) + 0.5)
-                radios.set_popularity(self._album_artist, popularity)
+                radios.set_popularity(self._album_artists[0], popularity)
         except Exception as e:
             print("Base::set_popularity(): %s" % e)
 
@@ -100,17 +100,21 @@ class Disc:
         self.db = Lp().albums
         self.album = album
         self.number = disc_number
+        self._track_ids = []
 
     @property
-    def tracks_ids(self):
+    def track_ids(self):
         """
             Get all tracks ids of the disc
 
             @return list of int
         """
-        return self.db.get_disc_tracks(self.album.id,
-                                       self.album.genre_ids,
-                                       self.number)
+        if not self._track_ids:
+            self._track_ids = self.db.get_disc_tracks(self.album.id,
+                                                      self.album.genre_ids,
+                                                      self.album.artist_ids,
+                                                      self.number)
+        return self._track_ids
 
     @property
     def tracks(self):
@@ -119,17 +123,17 @@ class Disc:
 
             @return list of Track
         """
-        return [Track(id) for id in self.tracks_ids]
+        return [Track(id) for id in self.track_ids]
 
 
 class Album(Base):
     """
         Represent an album
     """
-    FIELDS = ['name', 'artist_name', 'artist_id', 'year', 'path', 'duration']
-    DEFAULTS = ['', '', None, '', '', 0]
+    FIELDS = ['name', 'artists', 'artist_ids', 'year', 'path', 'duration']
+    DEFAULTS = ['', '', [], '', '', 0]
 
-    def __init__(self, album_id=None, genre_ids=[]):
+    def __init__(self, album_id=None, genre_ids=[], artist_ids=[]):
         """
             Init album
             @param album_id as int
@@ -138,12 +142,13 @@ class Album(Base):
         Base.__init__(self, Lp().albums)
         self.id = album_id
         self.genre_ids = genre_ids
+        # Use artist ids from db else
+        if artist_ids:
+            self.artist_ids = artist_ids
 
-    def set_genre(self, genre_ids):
+    def set_genres(self, genre_ids):
         """
-            Change current genre to lookup
-            tracks
-
+            Set album genres
             @param genre_ids as [int]
             @return None
         """
@@ -151,6 +156,14 @@ class Album(Base):
         self._tracks_ids = None
         self._tracks = None
 
+    def set_artists(self, artist_ids):
+        """
+            Set album artists
+        """
+        self.artist_ids = artist_ids
+        self._track_ids = None
+        self._tracks = None
+
     @property
     def title(self):
         """
@@ -167,7 +180,8 @@ class Album(Base):
         """
         if getattr(self, "_tracks_ids") is None:
             self._tracks_ids = self.db.get_tracks(self.id,
-                                                  self.genre_ids)
+                                                  self.genre_ids,
+                                                  self.artist_ids)
         return self._tracks_ids
 
     @property
@@ -195,10 +209,10 @@ class Track(Base):
     """
         Represent a track
     """
-    FIELDS = ['name', 'album_id', 'album_artist_id',
-              'artist_ids', 'album_name', 'artist_names',
-              'genre_names', 'duration', 'number', 'path', 'position']
-    DEFAULTS = ['', None, None, [], '', '', '', 0.0, None, '', 0]
+    FIELDS = ['name', 'album_id', 'album_artist_ids',
+              'artist_ids', 'album_name', 'artists',
+              'genres', 'duration', 'number', 'path', 'position']
+    DEFAULTS = ['', None, [], [], '', '', '', 0.0, None, '', 0]
 
     def __init__(self, track_id=None):
         """
@@ -215,12 +229,15 @@ class Track(Base):
             @return str
         """
         name = escape(self.name)
-        if self.album.artist_id == Type.COMPILATIONS or\
-           len(self.artist_ids) > 1 or\
-           self.album.artist_id not in self.artist_ids:
-            if self.artist_names != self.album.artist_name:
-                name = "%s [<b>%s</b>]" % (name,
-                                           escape(self.artist_names))
+        artists = []
+        # Show all artists for compilations
+        if self.album.artist_ids[0] == Type.COMPILATIONS:
+            artists = self.artists
+        # Show only non album artist for albums (and only if one)
+        elif len(self.artists) > 1:
+            artists = [self.artists[1]] + ["..."]
+        if artists:
+            name = "<b>%s</b> - %s" % (escape(", ".join(artists)), name)
         return name
 
     @property
@@ -270,39 +287,22 @@ class Track(Base):
         return self.album.year
 
     @property
-    def album_artist(self):
+    def album_artists(self):
         """
-            Get track artist name
+            Get track album artists, can be != than album.artists as track
+            may not have any album (radio, externals, ...)
             @return str
         """
-        if getattr(self, "_album_artist") is None:
-            self._album_artist = Lp().artists.get_name(self.album_artist_id)
-        return self._album_artist
+        if getattr(self, "_album_artists") is None:
+            self._album_artists = self.album.artists
+        return self._album_artists
 
-    @property
-    def artist(self):
-        """
-            Get track artist(s) name(s)
-            Alias to Track.artist_names
-            @return str
-        """
-        return self.artist_names
-
-    @property
-    def genre(self):
-        """
-            Get track genres
-            Alias to Track.genre_names
-            @return str
-        """
-        return self.genre_names
-
-    def set_album_artist(self, name):
+    def set_album_artists(self, artists):
         """
             Set album artist
-            @param name as string
+            @param artists as [int]
         """
-        self._album_artist = name
+        self._album_artists = artists
 
     def set_uri(self, uri):
         """
@@ -322,5 +322,5 @@ class Track(Base):
             @param uri as string
         """
         self.id = Type.RADIOS
-        self._album_artist = name
+        self._album_artists = [name]
         self._uri = uri
