@@ -19,7 +19,9 @@ from lollypop.player_externals import ExternalsPlayer
 from lollypop.player_userplaylist import UserPlaylistPlayer
 from lollypop.radios import Radios
 from lollypop.objects import Track
-from lollypop.define import Lp, Type, NextContext
+from lollypop.define import Lp, Type, NextContext, DataPath
+
+from pickle import load
 
 
 class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
@@ -41,19 +43,54 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         ExternalsPlayer.__init__(self)
         self.update_crossfading()
 
+    @property
+    def current_track(self):
+        """
+            Current track
+        """
+        if self._queue_track is not None:
+            return self._queue_track
+        else:
+            return self._current_track
+
+    @property
+    def next_track(self):
+        """
+            Current track
+        """
+        return self._next_track
+
+    @property
+    def prev_track(self):
+        """
+            Current track
+        """
+        return self._prev_track
+
+    @property
+    def context(self):
+        """
+            Current track
+        """
+        return self._context
+
     def prev(self):
         """
             Play previous track
         """
-        if self.prev_track.id is not None:
-            self.load(self.prev_track)
+        if self._prev_track.id is not None:
+            self.load(self._prev_track)
+        else:
+            self.stop()
 
     def next(self):
         """
             Play next track
         """
-        if self.next_track.id is not None:
-            self.load(self.next_track)
+        if self._next_track.id is not None:
+            self.load(self._next_track)
+        else:
+            self.stop()
 
     def load(self, track, play=True):
         """
@@ -79,22 +116,31 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
             @param album as Album
         """
         self.shuffle_albums(False)
-        if not self._albums:
-            self._albums = [self.current_track.album.id]
-            self.context.genre_ids[self.current_track.album.id] = []
-            self._user_playlist = []
-            self._user_playlist_ids = []
-        # If album already exists, merge genres
+        # Add album to main playlist if nothing playing
+        # Do not start to play
+        if not self._albums and self._current_track.id is None:
+                self.load(album.tracks[0], False)
+                self._albums = [self._current_track.album.id]
+                self._context.genre_ids[self._current_track.album.id] = []
+                self._context.artist_ids[self._current_track.album.id] = []
+                self._user_playlist = []
+                self._user_playlist_ids = []
+        # If album already exists, merge genres/artists
         if album.id in self._albums:
-            genre_ids = self.context.genre_ids[album.id]
+            genre_ids = self._context.genre_ids[album.id]
             for genre_id in album.genre_ids:
                 if genre_id >= 0 and genre_id not in genre_ids:
-                    self.context.genre_ids[album.id].append(genre_id)
+                    self._context.genre_ids[album.id].append(genre_id)
+            artist_ids = self._context.artist_ids[album.id]
+            for artist_id in album.artist_ids:
+                if artist_id >= 0 and artist_id not in artist_ids:
+                    self._context.artist_ids[album.id].append(artist_id)
         else:
             self._albums.append(album.id)
-            self.context.genre_ids[album.id] = album.genre_ids
+            self._context.genre_ids[album.id] = album.genre_ids
+            self._context.artist_ids[album.id] = album.artist_ids
         self.shuffle_albums(True)
-        if self.current_track.id is not None and self.current_track.id > 0:
+        if self._current_track.id is not None and self._current_track.id > 0:
             self.set_next()
             self.set_prev()
         self.emit('album-added', album.id)
@@ -115,13 +161,21 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         """
         try:
             # Remove genre ids from context
-            genre_ids = self.context.genre_ids[album.id]
+            genre_ids = self._context.genre_ids[album.id]
             for genre_id in album.genre_ids:
                 if genre_id in genre_ids:
                     genre_ids.remove(genre_id)
-            if not genre_ids:
-                self.context.genre_ids.pop(album.id, None)
+            artist_ids = self._context.artist_ids[album.id]
+            # Remove artist ids from context
+            for artist_id in album.artist_ids:
+                if artist_id in artist_ids:
+                    artist_ids.remove(artist_id)
+            if not genre_ids or not artist_ids:
+                self._context.genre_ids.pop(album.id, None)
+                self._context.artist_ids.pop(album.id, None)
                 self._albums.remove(album.id)
+                if album.id in self._albums_backup:
+                    self._albums_backup.remove(album.id)
             self.set_prev()
             self.set_next()
         except Exception as e:
@@ -133,28 +187,45 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
             @param album id as int
             @return genre ids as [int]
         """
-        if album_id in self.context.genre_ids.keys():
-            return self.context.genre_ids[album_id]
+        if album_id in self._context.genre_ids.keys():
+            return self._context.genre_ids[album_id]
         else:
             return []
 
-    def is_album_present(self, album):
+    def get_artist_ids(self, album_id):
         """
-            Check if album is present in albums, with context
+            Return artist ids for album
+            @param album id as int
+            @return genre ids as [int]
+        """
+        if album_id in self._context.artist_ids.keys():
+            return self._context.artist_ids[album_id]
+        else:
+            return []
+
+    def has_album(self, album):
+        """
+            Check if player has album
             @param album as Album
             @return bool
         """
-        is_present = True
+        is_genres = True
+        is_artists = True
         if album.id in self._albums:
             for genre_id in album.genre_ids:
-                if genre_id >= 0 and\
-                        album.id in self.context.genre_ids.keys() and\
-                        self.context.genre_ids[album.id] and\
-                        genre_id not in self.context.genre_ids[album.id]:
-                    is_present = False
+                if album.id in self._context.genre_ids.keys() and\
+                   self._context.genre_ids[album.id] and\
+                   genre_id not in self._context.genre_ids[album.id]:
+                    is_genres = False
+            for artist_id in album.artist_ids:
+                if album.id in self._context.artist_ids.keys() and\
+                   self._context.artist_ids[album.id] and\
+                   artist_id not in self._context.artist_ids[album.id]:
+                    is_artists = False
         else:
-            is_present = False
-        return is_present
+            is_genres = False
+            is_artists = False
+        return is_genres and is_artists
 
     def play_album(self, album):
         """
@@ -168,16 +239,21 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         self._user_playlist = []
         self._user_playlist_ids = []
         if Lp().settings.get_value('repeat'):
-            self.context.next = NextContext.NONE
+            self._context.next = NextContext.NONE
         else:
-            self.context.next = NextContext.STOP_ALL
-        self.context.genre_ids = {}
-        self.context.genre_ids[album.id] = []
+            self._context.next = NextContext.STOP_ALL
+        self._context.genre_ids = {}
+        self._context.artist_ids = {}
+        self._context.genre_ids[album.id] = []
+        self._context.artist_ids[album.id] = []
         for genre_id in album.genre_ids:
             if genre_id >= 0:
-                self.context.genre_ids[album.id].append(genre_id)
-        self.context.prev_track = Track()
-        self.context.next_track = Track()
+                self._context.genre_ids[album.id].append(genre_id)
+        for artist_id in album.artist_ids:
+            if artist_id >= 0:
+                self._context.artist_ids[album.id].append(artist_id)
+        self._context.prev_track = Track()
+        self._context.next_track = Track()
         self.load(album.tracks[0])
         self._albums = [album.id]
 
@@ -191,12 +267,11 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         # Invalid track
         if track_id is None:
             return
-        album = Track(track_id).album
-        album.set_genre(genre_ids)
         self._albums = []
-        self.context.genre_ids = {}
-        self.context.prev_track = Track()
-        self.context.next_track = Track()
+        self._context.genre_ids = {}
+        self._context.aritst_ids = {}
+        self._context.prev_track = Track()
+        self._context.next_track = Track()
         ShufflePlayer.reset_history(self)
 
         # We are not playing a user playlist anymore
@@ -205,11 +280,16 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         # We are in all artists
         if (genre_ids and genre_ids[0] == Type.ALL) or\
            (artist_ids and artist_ids[0] == Type.ALL):
-            if artist_ids and artist_ids[0] != Type.ALL:
+            # Genres: all, Artists: compilations
+            if artist_ids and artist_ids[0] == Type.COMPILATIONS:
+                self._albums += Lp().albums.get_compilations()
+            # Genres: all, Artists: ids
+            elif artist_ids and artist_ids[0] != Type.ALL:
                 self._albums += Lp().albums.get_ids(artist_ids)
+            # Genres: all, Artists: all
             else:
                 if Lp().settings.get_value('show-compilations'):
-                    self._albums = Lp().albums.get_compilations()
+                    self._albums += Lp().albums.get_compilations()
                 self._albums += Lp().albums.get_ids()
         # We are in populars view, add popular albums
         elif genre_ids and genre_ids[0] == Type.POPULARS:
@@ -227,23 +307,31 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         else:
             # If we are not in compilation view and show compilation is on,
             # add compilations
-            if not artist_ids and Lp().settings.get_value('show-compilations'):
+            if artist_ids and artist_ids[0] == Type.COMPILATIONS:
                 self._albums += Lp().albums.get_compilations(genre_ids)
-            self._albums += Lp().albums.get_ids(artist_ids, genre_ids)
+            else:
+                if not artist_ids and\
+                        Lp().settings.get_value('show-compilations'):
+                    self._albums += Lp().albums.get_compilations(genre_ids)
+                self._albums += Lp().albums.get_ids(artist_ids, genre_ids)
 
         if Lp().settings.get_value('repeat'):
-            self.context.next = NextContext.NONE
+            self._context.next = NextContext.NONE
         else:
-            self.context.next = NextContext.STOP_ALL
+            self._context.next = NextContext.STOP_ALL
         # We do not store genre_ids for ALL/POPULARS/...
         if genre_ids and genre_ids[0] < 0:
             genre_ids = []
         # Set context for each album
         for album_id in self._albums:
-            self.context.genre_ids[album_id] = []
+            self._context.genre_ids[album_id] = []
+            self._context.artist_ids[album_id] = []
             for genre_id in genre_ids:
                 if genre_id >= 0:
-                    self.context.genre_ids[album_id].append(genre_id)
+                    self._context.genre_ids[album_id].append(genre_id)
+            for artist_id in artist_ids:
+                if artist_id >= 0:
+                    self._context.artist_ids[album_id].append(artist_id)
         # Shuffle album list if needed
         self.shuffle_albums(True)
 
@@ -259,39 +347,43 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
             Clear all albums
         """
         self._albums = []
-        self.context.next = NextContext.STOP_TRACK
+        self._context.next = NextContext.STOP_TRACK
 
-    def get_current_artist(self):
+    def get_current_artists(self):
         """
             Get current artist
             @return artist as string
         """
-        artist_id = self.current_track.album_artist_id
-        if artist_id == Type.COMPILATIONS:
-            artist = self.current_track.artist
+        artist_ids = self._current_track.album.artist_ids
+        if artist_ids[0] == Type.COMPILATIONS:
+            artists = ", ".join(self._current_track.artists)
         else:
-            artist = self.current_track.album_artist
-        return artist
+            artists = ", ".join(self._current_track.album_artists)
+        return artists
 
     def restore_state(self):
         """
             Restore player state
         """
-        if Lp().settings.get_value('save-state'):
-            track_id = Lp().settings.get_value('track-id').get_int32()
-            playlist_ids = Lp().settings.get_value('playlist-ids')
-            if playlist_ids and playlist_ids[0] == Type.RADIOS:
-                radios = Radios()
-                track = Track()
-                name = radios.get_name(track_id)
-                url = radios.get_url(name)
-                track.set_radio(name, url)
-                self.load(track)
-            elif Lp().tracks.get_path(track_id) != "":
-                track = Track(track_id)
-                self._load_track(track)
-                if playlist_ids:
-                    try:
+        try:
+            if Lp().settings.get_value('save-state'):
+                track_id = load(open(DataPath + "/track_id.bin", "rb"))
+                playlist_ids = load(open(DataPath + "/playlist_ids.bin",
+                                    "rb"))
+                if playlist_ids and playlist_ids[0] == Type.RADIOS:
+                    radios = Radios()
+                    track = Track()
+                    name = radios.get_name(track_id)
+                    url = radios.get_url(name)
+                    track.set_radio(name, url)
+                    self.load(track)
+                elif Lp().tracks.get_path(track_id) != "":
+                    track = Track(track_id)
+                    self._load_track(track)
+                    # We set this initial state
+                    # because seek while failed otherwise
+                    self.pause()
+                    if playlist_ids:
                         pids = []
                         for playlist_id in playlist_ids:
                             pids.append(int(playlist_id))
@@ -306,29 +398,41 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
                             elif playlist_id == Type.RANDOMS:
                                 tracks = Lp().tracks.get_randoms()
                             else:
-                                tracks = Lp().playlists.get_tracks_ids(
+                                tracks = Lp().playlists.get_track_ids(
                                                                    playlist_id)
                             for track_id in tracks:
                                 if track_id not in track_ids:
                                     track_ids.append(track_id)
                             self.populate_user_playlist_by_tracks(track_ids,
                                                                   pids)
-                    except:
-                        pass  # User set non int in gsettings
+                    else:
+                        self._albums = load(open(
+                                            DataPath + "/albums.bin",
+                                            "rb"))
+                        self.shuffle_albums(True)
+                        self._context.genre_ids = load(open(
+                                            DataPath + "/genre_ids.bin",
+                                            "rb"))
+                        self._context.artist_ids = load(open(
+                                            DataPath + "/artist_ids.bin",
+                                            "rb"))
+                    self.set_next()
+                    self.set_prev()
+                    if Lp().settings.get_value('repeat'):
+                        self._context.next = NextContext.NONE
+                    else:
+                        self._context.next = NextContext.STOP_ALL
                 else:
-                    self._albums = Lp().artists.get_albums(
-                                                         track.album_artist_id)
-                    for album_id in self._albums:
-                        self.context.genre_ids[album_id] = []
-                self.set_next()
-                self.set_prev()
-                if Lp().settings.get_value('repeat'):
-                    self.context.next = NextContext.NONE
-                else:
-                    self.context.next = NextContext.STOP_ALL
-                self.emit('current-changed')
-            else:
-                print("Player::restore_state(): track missing")
+                    print("Player::restore_state(): track missing")
+        except Exception as e:
+            print("Player::restore_state()", e)
+
+    def set_next_context(self, value):
+        """
+            Set next context
+            @param context as int
+        """
+        self._context.next = value
 
     def set_party(self, party):
         """
@@ -345,33 +449,24 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         """
             Set previous track
         """
-        # Special case to return from queue
-        if self._queue and self.context.prev_track.id is None:
-            self.context.prev_track = self.current_track
-        elif not self._queue and self.context.prev_track.id is not None:
-            self.prev_track = self.context.prev_track
-            self.context.prev_track = Track()
-            self.emit('prev-changed')
-            return
-
         # Look at externals
-        self.prev_track = ExternalsPlayer.prev(self)
+        self._prev_track = ExternalsPlayer.prev(self)
 
         # Look at radio
-        if self.prev_track.id is None:
-            self.prev_track = RadioPlayer.prev(self)
+        if self._prev_track.id is None:
+            self._prev_track = RadioPlayer.prev(self)
 
         # Look at user playlist then
-        if self.prev_track.id is None:
-            self.prev_track = UserPlaylistPlayer.prev(self)
+        if self._prev_track.id is None:
+            self._prev_track = UserPlaylistPlayer.prev(self)
 
         # Look at shuffle
-        if self.prev_track.id is None:
-            self.prev_track = ShufflePlayer.prev(self)
+        if self._prev_track.id is None:
+            self._prev_track = ShufflePlayer.prev(self)
 
         # Get a linear track then
-        if self.prev_track.id is None:
-            self.prev_track = LinearPlayer.prev(self)
+        if self._prev_track.id is None:
+            self._prev_track = LinearPlayer.prev(self)
         self.emit('prev-changed')
 
     def set_next(self):
@@ -382,37 +477,28 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         # Reset finished context
         self._finished = NextContext.NONE
 
-        # Special case to return from queue
-        if self._queue and self.context.next_track.id is None:
-            self.context.next_track = self.next_track
-        elif not self._queue and self.context.next_track.id is not None:
-            self.next_track = self.context.next_track
-            self.context.next_track = Track()
-            self.emit('next-changed')
-            return
-
         # Look at externals
-        self.next_track = ExternalsPlayer.next(self)
+        self._next_track = ExternalsPlayer.next(self)
 
         # Look at radio
-        if self.next_track.id is None:
-            self.next_track = RadioPlayer.next(self)
+        if self._next_track.id is None:
+            self._next_track = RadioPlayer.next(self)
 
         # Look first at user queue
-        if self.next_track.id is None:
-            self.next_track = QueuePlayer.next(self)
+        if self._next_track.id is None:
+            self._next_track = QueuePlayer.next(self)
 
         # Look at user playlist then
-        if self.next_track.id is None:
-            self.next_track = UserPlaylistPlayer.next(self)
+        if self._next_track.id is None:
+            self._next_track = UserPlaylistPlayer.next(self)
 
         # Get a random album/track then
-        if self.next_track.id is None:
-            self.next_track = ShufflePlayer.next(self)
+        if self._next_track.id is None:
+            self._next_track = ShufflePlayer.next(self)
 
         # Get a linear track then
-        if self.next_track.id is None:
-            self.next_track = LinearPlayer.next(self)
+        if self._next_track.id is None:
+            self._next_track = LinearPlayer.next(self)
 
         self.emit('next-changed')
 
@@ -434,11 +520,8 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         """
         if not Lp().scanner.is_locked():
             Lp().window.pulse(False)
-        if self.current_track.id >= 0:
+        if self._current_track.id >= 0:
             ShufflePlayer._on_stream_start(self, bus, message)
-        if self._queue and self.current_track.id == self._queue[0]:
-            self._queue.pop(0)
-            self.emit("queue-changed")
         self.set_next()
         self.set_prev()
         BinPlayer._on_stream_start(self, bus, message)
