@@ -15,7 +15,7 @@ from gi.repository import Gtk, GLib, Gdk, Pango, GObject
 from cgi import escape
 from gettext import gettext as _
 
-from lollypop.widgets_album_context import AlbumPopoverWidget
+from lollypop.pop_album import AlbumPopover
 from lollypop.view import LazyLoadingView
 from lollypop.define import Lp, ArtSize, NextContext
 from lollypop.objects import Album
@@ -43,9 +43,9 @@ class AlbumRow(Gtk.ListBoxRow):
         cover_height = AlbumRow.MARGIN * 2 +\
             ArtSize.MEDIUM * widget.get_scale_factor()
         if font_height > cover_height:
-            return font_height
+            return font_height + 2
         else:
-            return cover_height
+            return cover_height + 2
 
     def __init__(self, album_id, height):
         """
@@ -77,7 +77,8 @@ class AlbumRow(Gtk.ListBoxRow):
         grid = Gtk.Grid()
         grid.set_column_spacing(8)
         self._artist_label = Gtk.Label.new("<b>%s</b>" %
-                                           escape(self._album.artist_name))
+                                           escape(", ".join(
+                                                         self._album.artists)))
         self._artist_label.set_use_markup(True)
         self._artist_label.set_hexpand(True)
         self._artist_label.set_property('halign', Gtk.Align.START)
@@ -85,6 +86,7 @@ class AlbumRow(Gtk.ListBoxRow):
         self._title_label = Gtk.Label.new(self._album.name)
         self._title_label.set_ellipsize(Pango.EllipsizeMode.END)
         cover = Gtk.Image()
+        cover.get_style_context().add_class('small-cover-frame')
         surface = Lp().art.get_album_artwork(
                                         self._album,
                                         ArtSize.MEDIUM*self.get_scale_factor())
@@ -92,15 +94,14 @@ class AlbumRow(Gtk.ListBoxRow):
         cover.set_size_request(ArtSize.MEDIUM*self.get_scale_factor(),
                                ArtSize.MEDIUM*self.get_scale_factor())
         del surface
-        cover_frame = Gtk.Frame()
-        cover_frame.get_style_context().add_class('small-cover-frame')
-        cover_frame.set_property('valign', Gtk.Align.CENTER)
-        cover_frame.add(cover)
         self._play_indicator = Gtk.Image.new_from_icon_name(
                                                'media-playback-start-symbolic',
                                                Gtk.IconSize.MENU)
         delete_button = Gtk.Button.new_from_icon_name('user-trash-symbolic',
                                                       Gtk.IconSize.MENU)
+        # Here a hack to make old Gtk version support min-height css attribute
+        # min-height = 24px, borders = 2px
+        delete_button.set_property('height-request', 26)
         delete_button.get_image().set_opacity(0.2)
         delete_button.set_relief(Gtk.ReliefStyle.NONE)
         delete_button.get_style_context().add_class('menu-button')
@@ -113,7 +114,7 @@ class AlbumRow(Gtk.ListBoxRow):
         vgrid.add(self._title_label)
         grid.attach(self._artist_label, 1, 0, 1, 1)
         grid.attach(delete_button, 2, 0, 1, 2)
-        grid.attach(cover_frame, 0, 0, 1, 2)
+        grid.attach(cover, 0, 0, 1, 2)
         grid.attach(vgrid, 1, 1, 1, 1)
         row_widget.add(grid)
         self.add(row_widget)
@@ -231,10 +232,19 @@ class AlbumRow(Gtk.ListBoxRow):
             @param button as Gtk.Button
         """
         if Lp().player.current_track.album.id == self._album.id:
-            Lp().player.context.next = NextContext.START_NEW_ALBUM
-            Lp().player.set_next()
-            Lp().player.next()
-        Lp().player.remove_album(self._album)
+            # If not last album, skip it
+            if len(Lp().player.get_albums()) > 1:
+                Lp().player.set_next_context(NextContext.START_NEW_ALBUM)
+                Lp().player.set_next()
+                Lp().player.next()
+                Lp().player.remove_album(self._album)
+            # remove it and stop playback by going to next track
+            else:
+                Lp().player.remove_album(self._album)
+                Lp().player.set_next()
+                Lp().player.next()
+        else:
+            Lp().player.remove_album(self._album)
         self.destroy()
 
     def _on_query_tooltip(self, widget, x, y, keyboard, tooltip):
@@ -275,15 +285,16 @@ class AlbumsView(LazyLoadingView):
                                                     'edit-clear-all-symbolic',
                                                     Gtk.IconSize.MENU)
         self._clear_button.set_relief(Gtk.ReliefStyle.NONE)
+        self._clear_button.set_tooltip_text(_("Clear albums"))
+        self._clear_button.set_sensitive(False)
         self._clear_button.connect('clicked', self._on_clear_clicked)
         self._jump_button = Gtk.Button.new_from_icon_name(
                                                     'go-jump-symbolic',
                                                     Gtk.IconSize.MENU)
         self._jump_button.set_relief(Gtk.ReliefStyle.NONE)
         self._jump_button.connect('clicked', self._on_jump_clicked)
-        # FIXME Translation breakage
-        text = _("Albums playing:")
-        label = Gtk.Label.new("<b>"+text[:-1]+"</b>")
+        self._jump_button.set_tooltip_text(_("Go to current track"))
+        label = Gtk.Label.new("<b>"+_("Playing albums")+"</b>")
         label.set_use_markup(True)
         label.set_hexpand(True)
         label.set_margin_start(2)
@@ -328,6 +339,7 @@ class AlbumsView(LazyLoadingView):
             child.destroy()
         if clear_albums:
             Lp().player.clear_albums()
+        self._clear_button.set_sensitive(False)
 
     def _add_items(self, items, prev_album_id=None):
         """
@@ -347,8 +359,6 @@ class AlbumsView(LazyLoadingView):
                 self._viewport.add(self._view)
             if Lp().player.current_track.album.id in Lp().player.get_albums():
                 self._jump_button.set_sensitive(True)
-                self._jump_button.set_tooltip_text(
-                                          Lp().player.current_track.album.name)
 
     def _row_for_album_id(self, album_id):
         """
@@ -398,9 +408,6 @@ class AlbumsView(LazyLoadingView):
             Show tracks in a popover
             @param player object
         """
-        if self._jump_button.get_sensitive():
-            self._jump_button.set_tooltip_text(
-                                          Lp().player.current_track.album.name)
         for child in self._view.get_children():
             child.show_play_indicator(child.get_id() ==
                                       Lp().player.current_track.album.id)
@@ -412,7 +419,6 @@ class AlbumsView(LazyLoadingView):
         """
         if row.get_id() != Lp().player.current_track.album.id:
             self._jump_button.set_sensitive(False)
-            self._jump_button.set_tooltip_text('')
         self._clear_button.set_sensitive(len(self._view.get_children()) != 0)
 
     def _on_row_activated(self, widget, row):
@@ -422,17 +428,17 @@ class AlbumsView(LazyLoadingView):
             @param row as AlbumRow
         """
         genre_ids = Lp().player.get_genre_ids(row.get_id())
+        artist_ids = Lp().player.get_artist_ids(row.get_id())
         # TODO Remove this later
         if Gtk.get_minor_version() > 16:
-            popover = AlbumPopoverWidget(
+            popover = AlbumPopover(
                                      row.get_id(),
                                      genre_ids,
-                                     [],
-                                     False)
+                                     [])
             popover.set_relative_to(row)
             popover.show()
         else:
-            album = Album(row.get_id(), genre_ids)
+            album = Album(row.get_id(), genre_ids, artist_ids)
             Lp().player.load(album.tracks[0])
 
     def _on_jump_clicked(self, widget):
