@@ -42,16 +42,14 @@ class QueueRow(Gtk.ListBoxRow):
         self._grid.set_column_spacing(5)
         self._row_widget.add(self._grid)
         self._cover = Gtk.Image()
-        self._cover_frame = Gtk.Frame()
-        self._cover_frame.set_shadow_type(Gtk.ShadowType.NONE)
-        self._cover_frame.set_property('halign', Gtk.Align.CENTER)
-        self._cover_frame.set_property('valign', Gtk.Align.CENTER)
-        self._cover_frame.get_style_context().add_class('small-cover-frame')
-        self._cover_frame.add(self._cover)
+        self._cover.set_no_show_all(True)
+        self._cover.set_property('halign', Gtk.Align.CENTER)
+        self._cover.set_property('valign', Gtk.Align.CENTER)
+        self._cover.get_style_context().add_class('small-cover-frame')
         # We force width with a Box
         box = Gtk.Box()
         box.set_homogeneous(True)
-        box.add(self._cover_frame)
+        box.add(self._cover)
         box.set_property('width-request', ArtSize.MEDIUM+2)
         box.show()
         self._title_label = Gtk.Label()
@@ -63,11 +61,14 @@ class QueueRow(Gtk.ListBoxRow):
         self._menu_button = Gtk.Button.new_from_icon_name(
                                                          'user-trash-symbolic',
                                                          Gtk.IconSize.MENU)
+        # Here a hack to make old Gtk version support min-height css attribute
+        # min-height = 24px, borders = 2px
+        self._menu_button.set_property('height-request', 26)
         self._menu_button.set_relief(Gtk.ReliefStyle.NONE)
         self._menu_button.get_style_context().add_class('menu-button')
         self._menu_button.get_style_context().add_class('track-menu-button')
         self._menu_button.get_image().set_opacity(0.2)
-        self._menu_button.set_property('valign', Gtk.Align.CENTER)
+        self._menu_button.set_property('valign', Gtk.Align.END)
         self._menu_button.show()
         self._menu_button.connect('clicked', self._on_delete_clicked)
         self._grid.add(box)
@@ -122,7 +123,8 @@ class QueueRow(Gtk.ListBoxRow):
         """
         track = Track(self._id)
         self._artist_label.set_markup(
-                                  "<b>"+escape(track.album.artist_name)+"</b>")
+                                 "<b>"+escape(
+                                        ", ".join(track.album.artists))+"</b>")
         self._album_label.set_text(track.album.name)
         # If we are listening to a compilation, prepend artist name
         title = escape(track.name)
@@ -141,10 +143,10 @@ class QueueRow(Gtk.ListBoxRow):
         """
         if surface is None:
             self._cover.clear()
-            self._cover_frame.set_shadow_type(Gtk.ShadowType.NONE)
+            self._cover.hide()
         else:
             self._cover.set_from_surface(surface)
-            self._cover_frame.set_shadow_type(Gtk.ShadowType.IN)
+            self._cover.show()
             del surface
 
 #######################
