@@ -30,7 +30,7 @@ def escape(str):
     """
     return "".join([c for c in str if
                     c.isalpha() or
-                    c.isdigit() or c in [' ', '.']]).rstrip()
+                    c.isdigit() or c in ['_', '-', ' ', '.']]).rstrip()
 
 
 def debug(str):
@@ -147,6 +147,14 @@ def is_loved(track_id):
                                        track_id)
 
 
+def remove_static_genres(genre_ids):
+    """
+        Remove static genre ids
+        @param genre ids as [int]
+    """
+    return [item for item in genre_ids if item >= 0]
+
+
 def set_loved(track_id, loved):
     """
         Add or remove track from loved playlist
@@ -181,17 +189,10 @@ def _set_loved_on_lastfm(track_id, loved):
     """
     # Love the track on lastfm
     if Gio.NetworkMonitor.get_default().get_network_available() and\
-            Lp().lastfm.is_auth():
+            Lp().lastfm.is_auth:
         title = Lp().tracks.get_name(track_id)
-        album_id = Lp().tracks.get_album_id(track_id)
-        artist_id = Lp().albums.get_artist_id(album_id)
-
-        if artist_id == Type.COMPILATIONS:
-            artist = Lp().tracks.get_artist_names(track_id)
-        else:
-            artist = Lp().artists.get_name(artist_id)
-
+        artists = ", ".join(Lp().tracks.get_artists(track_id))
         if loved:
-            Lp().lastfm.love(artist, title)
+            Lp().lastfm.love(artists, title)
         else:
-            Lp().lastfm.unlove(artist, title)
+            Lp().lastfm.unlove(artists, title)
