@@ -12,10 +12,10 @@
 
 from gi.repository import Gtk, Gdk
 
-from cgi import escape
+from gettext import gettext as _
 
 from lollypop.define import Lp
-from lollypop.pop_infos import InfosPopover
+from lollypop.pop_info import InfoPopover
 from lollypop.view_artist_albums import ArtistAlbumsView
 
 
@@ -41,12 +41,14 @@ class ArtistView(ArtistAlbumsView):
         builder.add_from_resource('/org/gnome/Lollypop/ArtistView.ui')
         builder.connect_signals(self)
         self._jump_button = builder.get_object('jump-button')
+        self._jump_button.set_tooltip_text(_("Go to current track"))
         self._spinner = builder.get_object('spinner')
         self.attach(builder.get_object('ArtistView'), 0, 0, 1, 1)
-        artists = ""
-        for artist_id in artist_ids:
-            artists += Lp().artists.get_name(artist_id) + ", "
-        builder.get_object('artist').set_label(artists[:-2])
+        if len(artist_ids) == 1:
+            artist = Lp().artists.get_name(artist_ids[0])
+        else:
+            artist = _("Many artists")
+        builder.get_object('artist').set_label(artist)
 
 #######################
 # PRIVATE             #
@@ -62,15 +64,8 @@ class ArtistView(ArtistAlbumsView):
                 break
         if found:
             self._jump_button.set_sensitive(True)
-            self._jump_button.set_tooltip_markup(
-                                      "<b>%s</b> - " %
-                                      escape(
-                                        Lp().player.current_track.artist_names
-                                        ) +
-                                      Lp().player.current_track.name)
         else:
             self._jump_button.set_sensitive(False)
-            self._jump_button.set_tooltip_text('')
 
     def _on_jump_button_clicked(self, widget):
         """
@@ -102,7 +97,7 @@ class ArtistView(ArtistAlbumsView):
             Change pointer on label
             @param eventbox as Gtk.EventBox
         """
-        if InfosPopover.should_be_shown() and\
+        if InfoPopover.should_be_shown() and\
                 self._artist_id is not None:
             eventbox.get_window().set_cursor(Gdk.Cursor(Gdk.CursorType.HAND1))
 
@@ -112,8 +107,8 @@ class ArtistView(ArtistAlbumsView):
             @param eventbox as Gtk.EventBox
             @param event as Gdk.Event
         """
-        if InfosPopover.should_be_shown() and\
+        if InfoPopover.should_be_shown() and\
                 self._artist_id is not None:
-            pop = InfosPopover(self._artist_id, False)
+            pop = InfoPopover([self._artist_id], False)
             pop.set_relative_to(eventbox)
             pop.show()
