@@ -12,9 +12,10 @@
 
 from gi.repository import Gtk, GLib, GObject
 
-from lollypop.view import LazyLoadingView, View
+from lollypop.view import LazyLoadingView
 from lollypop.view_container import ViewContainer
 from lollypop.define import Lp, Type
+from lollypop.objects import Track
 from lollypop.widgets_album import AlbumDetailedWidget
 
 
@@ -26,15 +27,17 @@ class ArtistAlbumsView(LazyLoadingView):
         'populated': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
-    def __init__(self, artist_ids, genre_ids):
+    def __init__(self, artist_ids, genre_ids, show_cover=True):
         """
             Init ArtistAlbumsView
             @param artist ids as [int]
             @param genre ids as [int]
+            @param show cover as bool
         """
         LazyLoadingView.__init__(self)
         self._artist_ids = artist_ids
         self._genre_ids = genre_ids
+        self._show_cover = show_cover
         self._albums_count = 0
 
         self._albumbox = Gtk.Grid()
@@ -75,6 +78,17 @@ class ArtistAlbumsView(LazyLoadingView):
         self._lazy_queue = []
         LazyLoadingView.stop(self)
 
+    @property
+    def requested_height(self):
+        """
+            Requested height for children
+            @return height as int
+        """
+        height = 0
+        for child in self._albumbox.get_children():
+            height += child.requested_height
+        return height
+
 #######################
 # PRIVATE             #
 #######################
@@ -116,7 +130,8 @@ class ArtistAlbumsView(LazyLoadingView):
             album_id = albums.pop(0)
             widget = AlbumDetailedWidget(album_id,
                                          self._genre_ids,
-                                         self._artist_ids)
+                                         self._artist_ids,
+                                         self._show_cover)
             self._lazy_queue.append(widget)
             # Not needed if only one album
             if self._albums_count == 1:
@@ -157,21 +172,25 @@ class CurrentArtistAlbumsView(ViewContainer):
         """
         ViewContainer.__init__(self, 1000)
         self.connect('destroy', self._on_destroy)
-        self._current = (Type.NONE, Type.NONE)
+        self._track = Track()
 
-    def populate(self, artist_id, album_id):
+    def populate(self, track):
         """
             Populate the view
+            @param track as Track
             @thread safe
         """
-        if (artist_id, album_id) != self._current:
-            self._current = (artist_id, album_id)
-            if album_id == Type.NONE:
-                albums = self._get_albums(artist_id)
-            else:
-                albums = [album_id]
+        if track.album.artist_ids != self._track.album.artist_ids:
+            self._track = track
+            albums = self._get_albums()
             GLib.idle_add(self._populate, albums)
 
+    def stop(self):
+        """
+            Stop loading
+        """
+        self.get_visible_child().stop()
+
     def jump_to_current(self):
         """
             Jump to current track
@@ -193,42 +212,21 @@ class CurrentArtistAlbumsView(ViewContainer):
             Populate view and make it visible
             @param albums as [albums ids as int]
         """
-        # Add a loading indicator
-        view = View()
-        spinner = Gtk.Spinner()
-        spinner.set_hexpand(True)
-        spinner.set_property('halign', Gtk.Align.END)
-        spinner.start()
-        view.add(spinner)
-        view.show_all()
+        # Populate artist albums view
+        view = ArtistAlbumsView(self._track.album.artist_ids, [])
+        view.show()
+        view.populate(albums)
         self.add(view)
         self.set_visible_child(view)
         self.clean_old_views(view)
 
-        # Populate artist albums view
-        view = ArtistAlbumsView([self._current[0]], [])
-        view.connect('populated', self._on_populated, spinner)
-        view.show()
-        view.populate(albums)
-
-    def _get_albums(self, artist_id):
+    def _get_albums(self):
         """
             Get albums
             @return album ids as [int]
         """
-        if artist_id == Type.COMPILATIONS:
-            albums = [Lp().player.current_track.album_id]
+        if self._track.album.artist_ids[0] == Type.COMPILATIONS:
+            albums = [self._track.album.id]
         else:
-            albums = Lp().artists.get_albums(artist_id)
+            albums = Lp().artists.get_albums(self._track.album.artist_ids)
         return albums
-
-    def _on_populated(self, view, spinner):
-        """
-            Show the view
-            @param view as View
-            @param spinner as Gtk.Spinner
-        """
-        spinner.stop()
-        self.add(view)
-        self.set_visible_child(view)
-        self.clean_old_views(view)
