@@ -20,7 +20,7 @@ from lollypop.widgets_album import AlbumWidget
 from lollypop.pop_radio import RadioPopover
 
 
-class RadioWidget(Gtk.Frame, AlbumWidget):
+class RadioWidget(Gtk.FlowBoxChild, AlbumWidget):
     """
         Widget with radio cover and title
     """
@@ -31,8 +31,7 @@ class RadioWidget(Gtk.Frame, AlbumWidget):
             @param name as string
             @param radios_manager as RadiosManager
         """
-        Gtk.Frame.__init__(self)
-        self.set_shadow_type(Gtk.ShadowType.NONE)
+        Gtk.FlowBoxChild.__init__(self)
         self.get_style_context().add_class('loading')
         self._name = name
         self._cover = None
@@ -47,66 +46,30 @@ class RadioWidget(Gtk.Frame, AlbumWidget):
         self._widget = Gtk.EventBox()
         self._widget.connect('enter-notify-event', self._on_enter_notify)
         self._widget.connect('leave-notify-event', self._on_leave_notify)
-        grid = Gtk.Grid()
-        grid.set_orientation(Gtk.Orientation.VERTICAL)
-        frame = Gtk.Frame()
-        frame.set_property('halign', Gtk.Align.CENTER)
-        frame.get_style_context().add_class('cover-frame')
-        self._color = Gtk.Frame()
-        self._color.get_style_context().add_class('cover-frame-border')
         self._cover = Gtk.Image()
+        self._cover.set_property('halign', Gtk.Align.CENTER)
         self._cover.set_size_request(ArtSize.BIG, ArtSize.BIG)
-        self._cover.get_style_context().add_class('white')
         self._title_label = Gtk.Label()
         self._title_label.set_ellipsize(Pango.EllipsizeMode.END)
         self._title_label.set_property('halign', Gtk.Align.CENTER)
         self._title_label.set_text(self._name)
-        self._widget.add(grid)
-        grid.add(frame)
+        self._overlay = Gtk.Overlay()
+        frame = Gtk.Frame()
+        frame.get_style_context().add_class('cover-frame')
+        frame.add(self._cover)
+        self._overlay.add(frame)
+        grid = Gtk.Grid()
+        grid.set_orientation(Gtk.Orientation.VERTICAL)
+        self._overlay.get_style_context().add_class('white')
+        grid.add(self._overlay)
         grid.add(self._title_label)
-        frame.add(self._color)
-        overlay = Gtk.Overlay.new()
-        overlay.get_style_context().add_class('white')
-        # Play button
-        play_event = Gtk.EventBox()
-        play_event.set_property('has-tooltip', True)
-        play_event.set_tooltip_text(_("Play"))
-        play_event.set_hexpand(True)
-        play_event.set_property('valign', Gtk.Align.CENTER)
-        play_event.set_property('halign', Gtk.Align.CENTER)
-        play_event.connect('realize', self._on_eventbox_realize)
-        play_event.connect('button-press-event',
-                           self._on_play_press_event)
-        self._play_button = Gtk.Image.new_from_icon_name(
-                                           'media-playback-start-symbolic',
-                                           Gtk.IconSize.DND)
-        self._play_button.set_opacity(0)
-        # Edit button
-        edit_event = Gtk.EventBox()
-        edit_event.set_property('has-tooltip', True)
-        edit_event.set_tooltip_text(_("Change artwork"))
-        edit_event.set_property('halign', Gtk.Align.END)
-        edit_event.connect('realize', self._on_eventbox_realize)
-        edit_event.connect('button-press-event',
-                           self._on_edit_press_event)
-        edit_event.set_property('valign', Gtk.Align.END)
-        edit_event.set_property('halign', Gtk.Align.END)
-        self._edit_button = Gtk.Image.new_from_icon_name(
-                                           'document-properties-symbolic',
-                                           Gtk.IconSize.BUTTON)
-        self._edit_button.set_opacity(0)
-        overlay.add(self._cover)
-        overlay.add_overlay(play_event)
-        overlay.add_overlay(edit_event)
-        self._color.add(overlay)
+        self._widget.add(grid)
+        self._widget.set_property('halign', Gtk.Align.CENTER)
+        self._widget.set_property('valign', Gtk.Align.CENTER)
         self.add(self._widget)
         self.set_cover()
         self.update_state()
-        self._widget.set_property('halign', Gtk.Align.CENTER)
-        self._widget.set_property('valign', Gtk.Align.CENTER)
         self.show_all()
-        play_event.add(self._play_button)
-        edit_event.add(self._edit_button)
 
     def set_sensitive(self, b):
         """
@@ -177,67 +140,72 @@ class RadioWidget(Gtk.Frame, AlbumWidget):
         if self._cover is None:
             return
         selected = Lp().player.current_track.id == Type.RADIOS and\
-            self._name == Lp().player.current_track.album_artist
+            self._name == Lp().player.current_track.album_artists[0]
         if selected:
-            self._color.get_style_context().add_class(
+            self._overlay.get_style_context().add_class(
                                                     'cover-frame-selected')
         else:
-            self._color.get_style_context().remove_class(
+            self._overlay.get_style_context().remove_class(
                                                     'cover-frame-selected')
 
+    def set_overlay(self, set):
+        """
+            Set overlay
+            @param set as bool
+        """
+        if self._show_overlay == set:
+            return
+        if set:
+            # Play button
+            self._play_event = Gtk.EventBox()
+            self._play_event.set_property('has-tooltip', True)
+            self._play_event.set_tooltip_text(_("Play"))
+            self._play_event.set_hexpand(True)
+            self._play_event.set_property('valign', Gtk.Align.CENTER)
+            self._play_event.set_property('halign', Gtk.Align.CENTER)
+            self._play_event.connect('realize', self._on_eventbox_realize)
+            self._play_event.connect('button-press-event',
+                                     self._on_play_press_event)
+            self._play_button = Gtk.Image.new_from_icon_name(
+                                               'media-playback-start-symbolic',
+                                               Gtk.IconSize.DND)
+            self._play_button.set_opacity(0)
+            # Edit button
+            self._artwork_event = Gtk.EventBox()
+            self._artwork_event.set_margin_bottom(5)
+            self._artwork_event.set_margin_end(5)
+            self._artwork_event.set_property('has-tooltip', True)
+            self._artwork_event.set_tooltip_text(_("Modify radio"))
+            self._artwork_event.set_property('halign', Gtk.Align.END)
+            self._artwork_event.connect('realize', self._on_eventbox_realize)
+            self._artwork_event.connect('button-press-event',
+                                        self._on_edit_press_event)
+            self._artwork_event.set_property('valign', Gtk.Align.END)
+            self._artwork_event.set_property('halign', Gtk.Align.END)
+            self._artwork_button = Gtk.Image.new_from_icon_name(
+                                               'document-properties-symbolic',
+                                               Gtk.IconSize.BUTTON)
+            self._artwork_button.set_opacity(0)
+            self._play_event.add(self._play_button)
+            self._artwork_event.add(self._artwork_button)
+            self._overlay.add_overlay(self._play_event)
+            self._overlay.add_overlay(self._artwork_event)
+            self._overlay.show_all()
+            AlbumWidget.set_overlay(self, True)
+        else:
+            AlbumWidget.set_overlay(self, False)
+            self._play_event.destroy()
+            self._play_event = None
+            self._play_button.destroy()
+            self._play_button = None
+            self._artwork_event.destroy()
+            self._artwork_event = None
+            self._artwork_button.destroy()
+            self._artwork_button = None
+
 #######################
 # PRIVATE             #
 #######################
-    def _on_enter_notify(self, widget, event):
-        """
-            Show special buttons
-            @param widget as Gtk.Widget
-            @param event es Gdk.Event
-        """
-        if self._play_button.get_opacity() == 0:
-            self._cover.get_style_context().add_class('hovereffect')
-            self._play_button.set_opacity(1)
-            self._play_button.get_style_context().add_class(
-                                                           self._rounded_class)
-            self._play_button.show()
-            self._edit_button.set_opacity(1)
-            self._edit_button.get_style_context().add_class(
-                                                           self._squared_class)
-            self._edit_button.show()
-
-    def _on_pop_cover_closed(self, widget):
-        """
-            Remove selected style
-            @param widget as Gtk.Popover
-        """
-        self._popover = False
-        self._play_button.set_opacity(0)
-        self._play_button.get_style_context().remove_class(self._rounded_class)
-        self._edit_button.set_opacity(0)
-        self._edit_button.get_style_context().remove_class(
-                                                           self._squared_class)
-
-    def _on_leave_notify(self, widget, event):
-        """
-            Hide special buttons
-            @param widget as Gtk.Widget
-            @param event es Gdk.Event
-        """
-        allocation = widget.get_allocation()
-        if event.x < 5 or\
-           event.x > allocation.width - 5 or\
-           event.y < 5 or\
-           event.y > allocation.height - 5:
-            self._cover.get_style_context().remove_class('hovereffect')
-            self._play_button.set_opacity(0)
-            self._play_button.hide()
-            self._play_button.get_style_context().remove_class(
-                                                           self._rounded_class)
-            self._edit_button.hide()
-            self._edit_button.set_opacity(0)
-            self._edit_button.get_style_context().remove_class(
-                                                           self._squared_class)
-
     def _on_play_press_event(self, widget, event):
         """
             Play radio
@@ -256,8 +224,9 @@ class RadioWidget(Gtk.Frame, AlbumWidget):
             @param: widget as Gtk.EventBox
             @param: event as Gdk.Event
         """
-        self._popover = True
         popover = RadioPopover(self._name, self._radios_manager)
         popover.set_relative_to(widget)
         popover.connect('closed', self._on_pop_cover_closed)
+        # Disable hidding overlay
+        self._show_overlay = False
         popover.show()
