@@ -36,13 +36,11 @@ class Row(Gtk.ListBoxRow):
         """
         # We do not use Gtk.Builder for speed reasons
         Gtk.ListBoxRow.__init__(self)
-        self.set_sensitive(False)
         self._track = Track(rowid)
         self._number = num
         self._indicator = IndicatorWidget(self._track.id)
         self.set_indicator(Lp().player.current_track.id == self._track.id,
                            utils.is_loved(self._track.id))
-        self.set_sensitive(True)
         self._row_widget = Gtk.EventBox()
         self._row_widget.connect('button-press-event', self._on_button_press)
         self._row_widget.connect('enter-notify-event', self._on_enter_notify)
@@ -66,14 +64,14 @@ class Row(Gtk.ListBoxRow):
         self._num_label.set_width_chars(4)
         self._num_label.get_style_context().add_class('dim-label')
         self.update_num_label()
-        self._menu_image = Gtk.Image.new()
-        self._menu_image.set_opacity(0.2)
         self._menu_button = Gtk.Button.new()
+        # Here a hack to make old Gtk version support min-height css attribute
+        # min-height = 24px, borders = 2px, we set directly on stack
+        # min-width = 24px, borders = 2px, padding = 8px
+        self._menu_button.set_size_request(34, 26)
         self._menu_button.set_relief(Gtk.ReliefStyle.NONE)
         self._menu_button.get_style_context().add_class('menu-button')
         self._menu_button.get_style_context().add_class('track-menu-button')
-        self._menu_button.set_image(self._menu_image)
-        self._menu_button.connect('clicked', self._on_button_clicked)
         self._grid.add(self._num_label)
         self._grid.add(self._title_label)
         self._grid.add(self._duration_label)
@@ -159,9 +157,12 @@ class Row(Gtk.ListBoxRow):
             @param widget as Gtk.Widget
             @param event as Gdk.Event
         """
-        if self._menu_image.get_pixbuf() is None:
-            self._menu_image.set_from_icon_name('open-menu-symbolic',
-                                                Gtk.IconSize.MENU)
+        if self._menu_button.get_image() is None:
+            image = Gtk.Image.new_from_icon_name('open-menu-symbolic',
+                                                 Gtk.IconSize.MENU)
+            image.set_opacity(0.2)
+            self._menu_button.set_image(image)
+            self._menu_button.connect('clicked', self._on_button_clicked)
             self._indicator.update_button()
 
     def _on_button_press(self, widget, event):
@@ -170,7 +171,7 @@ class Row(Gtk.ListBoxRow):
             @param widget as Gtk.Widget
             @param event as Gdk.Event
         """
-        if event.button == 3:
+        if event.button == 3 and Gtk.get_minor_version() > 16:
             window = widget.get_window()
             if window == event.window:
                 self._popup_menu(widget, event.x, event.y)
@@ -228,8 +229,10 @@ class Row(Gtk.ListBoxRow):
         """
         layout = self._title_label.get_layout()
         if layout.is_ellipsized():
-            label = self._title_label.get_label()
-            self._title_label.set_tooltip_markup(label)
+            self._title_label.set_tooltip_markup(self._track.formated_name())
+        elif len(self._track.artists) > 1:
+            self._title_label.set_tooltip_text(
+                                           ", ".join(self._track.artists[1:]))
         else:
             self._title_label.set_tooltip_text('')
 
@@ -260,25 +263,24 @@ class PlaylistRow(Row):
         self._grid.insert_column(1)
         self._grid.attach(self._indicator, 1, 1, 1, 2)
         self._cover = Gtk.Image()
-        self._cover_frame = Gtk.Frame()
-        self._cover_frame.set_shadow_type(Gtk.ShadowType.NONE)
-        self._cover_frame.set_property('halign', Gtk.Align.CENTER)
-        self._cover_frame.set_property('valign', Gtk.Align.CENTER)
-        self._cover_frame.get_style_context().add_class('small-cover-frame')
-        self._cover_frame.add(self._cover)
+        self._cover.set_property('halign', Gtk.Align.CENTER)
+        self._cover.set_property('valign', Gtk.Align.CENTER)
+        self._cover.get_style_context().add_class('small-cover-frame')
+        self._cover.set_no_show_all(True)
         # We force width with a Box
         box = Gtk.Box()
         box.set_homogeneous(True)
-        box.add(self._cover_frame)
+        box.add(self._cover)
         box.set_property('width-request', ArtSize.MEDIUM+2)
         self._grid.attach(box, 0, 0, 1, 2)
         self.show_all()
         self._header = Gtk.Grid()
         self._header.set_column_spacing(5)
         self._artist_label = Gtk.Label()
-        self._artist_label.set_markup("<b>" +
-                                      escape(self._track.album.artist_name) +
-                                      "</b>")
+        self._artist_label.set_markup(
+                                 "<b>" +
+                                 escape(", ".join(self._track.artists)) +
+                                 "</b>")
         self._artist_label.set_ellipsize(Pango.EllipsizeMode.END)
         self._artist_label.get_style_context().add_class('dim-label')
         self._album_label = Gtk.Label.new(self._track.album.name)
@@ -317,18 +319,18 @@ class PlaylistRow(Row):
             return
         self._show_headers = show
         if show:
-            self._cover_frame.set_shadow_type(Gtk.ShadowType.IN)
             self._cover.set_tooltip_text(self._track.album.name)
             surface = Lp().art.get_album_artwork(
                                         self._track.album,
                                         ArtSize.MEDIUM*self.get_scale_factor())
             self._cover.set_from_surface(surface)
+            self._cover.show()
             del surface
             self._header.show_all()
         else:
-            self._cover_frame.set_shadow_type(Gtk.ShadowType.NONE)
             self._cover.set_tooltip_text('')
             self._cover.clear()
+            self._cover.hide()
             self._header.hide()
 
 #######################
@@ -417,9 +419,8 @@ class TrackRow(Row):
         layout = Pango.Layout.new(ctx)
         layout.set_text("a", 1)
         font_height = int(layout.get_pixel_size()[1])
-        # Gtk.IconSize.MENU + .menu-button padding + border +
-        # image spacing(application.css)
-        menu_height = 16 + 8 + 2
+        # Button min height + borders (application.css)
+        menu_height = 24 + 2
         if font_height > menu_height:
             height = font_height
         else:
