@@ -16,6 +16,8 @@ from gi.repository import Gtk, WebKit2, GLib
 
 from urllib.parse import urlsplit
 
+from lollypop.define import OpenLink
+
 
 class WebView(Gtk.Stack):
     """
@@ -33,10 +35,13 @@ class WebView(Gtk.Stack):
         self.connect('destroy', self._on_destroy)
         self.set_transition_duration(500)
         self.set_transition_type(Gtk.StackTransitionType.CROSSFADE)
-        self._current = ''
+        self._current_domain = ''
+        self._url = ''
+        self._allowed_words = []
+        self._open_links = OpenLink.NEW
         builder = Gtk.Builder()
         # Use ressource from ArtistContent
-        builder.add_from_resource('/org/gnome/Lollypop/ArtistContent.ui')
+        builder.add_from_resource('/org/gnome/Lollypop/InfoContent.ui')
         self._view = WebKit2.WebView()
         self._spinner = builder.get_object('spinner')
         self.add_named(self._spinner, 'spinner')
@@ -47,7 +52,18 @@ class WebView(Gtk.Stack):
         # Private browsing make duckduckgo fail to switch translations
         if private:
             settings.set_property('enable-private-browsing', True)
+            settings.set_property('print-backgrounds', False)
+        settings.set_property('enable-smooth-scrolling', True)
         settings.set_property('enable-plugins', False)
+        settings.set_property('enable-fullscreen', False)
+        settings.set_property('enable-html5-database', False)
+        settings.set_property('enable-html5-local-storage', False)
+        settings.set_property('enable-media-stream', False)
+        settings.set_property('enable-mediasource', False)
+        settings.set_property('enable-offline-web-application-cache', False)
+        settings.set_property('enable-page-cache', False)
+        settings.set_property('enable-webaudio', False)
+        settings.set_property('enable-webgl', False)
         if mobile:
             settings.set_property('user-agent',
                                   "Mozilla/5.0 (Linux; Ubuntu 14.04;"
@@ -57,23 +73,56 @@ class WebView(Gtk.Stack):
         # FIXME TLS is broken in WebKit2, don't know how to fix this
         self._view.get_context().set_tls_errors_policy(
                                                 WebKit2.TLSErrorsPolicy.IGNORE)
-        self._view.connect('decide_policy', self._on_decide_policy)
+        self._view.connect('decide-policy', self._on_decide_policy)
+        self._view.connect('context-menu', self._on_context_menu)
         self._view.set_property('hexpand', True)
         self._view.set_property('vexpand', True)
         self._view.show()
 
-    def load(self, url):
+    @property
+    def url(self):
+        """
+            domain as str
+        """
+        return self._url
+
+    def add_word(self, word):
+        """
+            Add a word to allowed urls, only urls with this word will
+            get a navigation token
+        """
+        self._allowed_words.append(word)
+
+    def load(self, url, open_link):
         """
             Load url
             @param url as string
+            @param open link as OpenLink
         """
-        self._current = urlsplit(url)[1]
+        self._open_link = open_link
+        self._url = url
+        self._current_domain = self._get_domain(url)
         self._view.grab_focus()
         self._view.load_uri(url)
 
+    def stop(self):
+        """
+            Stop loading
+        """
+        self._view.stop_loading()
+
 #######################
 # PRIVATE             #
 #######################
+    def _get_domain(self, url):
+        """
+            Return domain for url
+            @param url as str
+        """
+        hostname = urlsplit(url)[1]
+        split = hostname.split('.')
+        return split[-2] + "." + split[-1]
+
     def _on_destroy(self, widget):
         """
             Destroy webkit view to stop any audio playback
@@ -97,21 +146,62 @@ class WebView(Gtk.Stack):
 
     def _on_decide_policy(self, view, decision, decision_type):
         """
-            Disallow navigation, launch in external browser
+            Navigation policy
             @param view as WebKit2.WebView
             @param decision as WebKit2.NavigationPolicyDecision
             @param decision_type as WebKit2.PolicyDecisionType
             @return bool
         """
-        if decision_type == WebKit2.PolicyDecisionType.NAVIGATION_ACTION:
-            if decision.get_navigation_action().get_navigation_type() ==\
-               WebKit2.NavigationType.LINK_CLICKED or (self._current not
-               in decision.get_navigation_action().get_request().get_uri() and
-               decision.get_navigation_action().get_request().get_uri() !=
-               'about:blank'):
-                decision.ignore()
-                GLib.spawn_command_line_async("xdg-open \"%s\"" %
-                                              decision.get_request().get_uri())
+        # Always accept response
+        if decision_type == WebKit2.PolicyDecisionType.RESPONSE:
+            decision.use()
+            return False
+
+        url = decision.get_navigation_action().get_request().get_uri()
+        # WTF is this?
+        if url == "about:blank":
+            decision.ignore()
             return True
+
+        # Refused non allowed words
+        found = False
+        for word in self._allowed_words:
+            if word in url:
+                found = True
+                break
+        if self._allowed_words and not found:
+            decision.ignore()
+            return True
+
+        # On clicked, if external wanted, launch user browser and stop
+        # If navigation not allowed, stop
+        if self._open_link == OpenLink.NEW and\
+                self._get_domain(url) != self._current_domain:
+            GLib.spawn_command_line_async("xdg-open \"%s\"" % url)
+            decision.ignore()
+            return True
+        elif self._open_link == OpenLink.NONE:
+            decision.ignore()
+            return True
+        # Use click, load page
+        elif decision.get_navigation_action().get_navigation_type() ==\
+                WebKit2.NavigationType.LINK_CLICKED:
+            decision.use()
+            return False
+        # If external domain, do not load
+        elif self._get_domain(url) != self._current_domain:
+            decision.ignore()
+            return True
+        self._current_domain = self._get_domain(url)
         decision.use()
         return False
+
+    def _on_context_menu(self, view, menu, event, hit):
+        """
+            No menu
+            @param view as WebKit2.WebView
+            @param menu as WebKit2.ContextMenu
+            @param event as Gdk.Event
+            @param hit as WebKit2.HitTestResult
+        """
+        return True
